@@ -266,10 +266,28 @@ class Find:
                         total_res["fetched_results"][k].update(v)
             else:
                 total_res["success"] = False
-                total_res["message"] = res["message"]
+                total_res["message"] = res["message"] if "message" in res else (res["error"] if "error" in res else "")
                 break
         return total_res
 
+    async def find_all_generator(self, page_size=1000):
+        max_page_size=1000
+        if page_size > max_page_size:
+            raise ValueError(f"El tamaño máximo permitido es {max_page_size}, pero se recibio {page_size}")
+        self.set_page_size(page_size)
+        self.set_page(1)
+        self.set_url(True)
+        query_compiled = self.query.compile()
+        data = await self.__then(query_compiled)
+        yield data
+        if data["success"]:
+            total_pages = data["total_pages"]
+            for i in range(1, total_pages):
+                query_aux = copy.deepcopy(query_compiled)
+                query_aux['page'] = i
+                data = await self.__then(query_aux)
+                yield data
+
     async def find_all_query(self, page_size=1000, max_in_parallel=2):
         self.set_page_size(page_size)
         self.set_page(1)
@@ -504,8 +522,28 @@ class SbxCore:
                 return await resp.json()
 
     async def upsert(self, model, data, let_null=False):
-        query = self.query_builder_to_insert(data, let_null).set_model(model).compile()
-        return await self.__then(query, self.is_update(data))
+        update_list = [item for item in data if "_KEY" in item]
+        insert_list = [item for item in data if "_KEY" not in item]
+        merged_results = {"success": True}
+        if update_list:
+            query_update = self.query_builder_to_insert(update_list, let_null).set_model(model).compile()
+            result_update = await self.__then(query_update, self.is_update(update_list))
+            merged_results['success'] = merged_results['success'] and result_update.get("success",False)
+            for k, v in result_update.items():
+                if k != "success":
+                    merged_results[k]=v
+        if insert_list:
+            query_insert = self.query_builder_to_insert(insert_list, let_null).set_model(model).compile()
+            result_insert= await self.__then(query_insert, False)
+            merged_results['success'] = merged_results['success'] and result_insert.get("success",False)
+            for k, v in result_insert.items():
+                if k != "success":
+                    if k in merged_results and isinstance(v,list):
+                        merged_results[k].extend(v)
+                    else:
+                        merged_results[k]=v
+        return merged_results
+
 
     async def __then(self, query_compiled, update):
         async with aiohttp.ClientSession() as session:
